//
//  tank
//
//  Import and Playlist Daemon for autoradio project
//
//
//  Copyright (C) 2017-2018 Christian Pointner <equinox@helsinki.at>
//
//  This file is part of tank.
//
//  tank is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  any later version.
//
//  tank is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with tank. If not, see <http://www.gnu.org/licenses/>.
//

package importer

import (
	"io"
	"sync/atomic"
	"time"
)

func (job *Job) initializeSource() (err error) {
	if job.Source.Scheme == SourceSchemeUpload {
		// the source will be attached using job.AttachSource() so all we need to do
		// is to wait for it to happen
		select {
		case <-job.Ctx.Done():
			return job.Ctx.Err()
		case <-job.subC.sourceAttached:
		}
		return
	}

	atomic.StoreUint32(&job.sourceSet, 1)

	switch job.Source.Scheme {
	case SourceSchemeFake:
		job.source = newJobSourceFake(job.Source)
	default:
		return ErrSourceNotSupported
	}

	close(job.subC.sourceAttached)
	return nil
}

//******* fake

type JobSourceFake uint64

func newJobSourceFake(srcURL SourceURL) *JobSourceFake {
	// TODO: parse size from srcURL
	src := JobSourceFake(10 * 1024 * 1024) // simulate a 10 MB file...
	return &src
}

func (src *JobSourceFake) Len() uint64 {
	return uint64(*src)
}

func (src *JobSourceFake) Read(p []byte) (n int, err error) {
	l := len(p)
	if *src > JobSourceFake(l) {
		*src = *src - JobSourceFake(l)
		time.Sleep(200 * time.Millisecond)
		return l, nil
	}
	l = int(*src)
	*src = 0
	return l, io.EOF
}

func (src *JobSourceFake) Done(result *JobSourceResult) {
	return
}

//******* Upload

type JobSourceUpload struct {
	len  uint64
	r    io.Reader
	done chan *JobSourceResult
}

func newJobSourceUpload(len uint64, r io.Reader) *JobSourceUpload {
	src := &JobSourceUpload{len: len, r: r}
	src.done = make(chan *JobSourceResult, 1)
	return src
}

func (src *JobSourceUpload) Len() uint64 {
	return src.len
}

func (src *JobSourceUpload) Read(p []byte) (n int, err error) {
	return src.r.Read(p)
}

func (src *JobSourceUpload) Done(result *JobSourceResult) {
	if result == nil {
		close(src.done)
		return
	}
	src.done <- result
}
